#include "cdburner.h"
#include "discutils.h"
#include "folderutils.h"
#include "setup.h"
#include "insertdiscdialog.h"

CDburner *burnprocess;
extern Setup *setup;
extern GtkWidget *mainWindow;

CDburner::CDburner(){
  texttagtable = gtk_text_tag_table_new();
  textbuffer = gtk_text_buffer_new(texttagtable); 
  theProcesses.setCDBurner(this);

  burnergui = new BurnerGUI();
  burnergui->burnDialog();

  setGtkItems(GTK_TEXT_VIEW(burnergui->process_textview), 
	      GTK_LABEL(burnergui->status_label), 
	      GTK_BUTTON(burnergui->burn_done_button), 
	      GTK_BUTTON(burnergui->burn_cancel_button), 
	      GTK_SCROLLED_WINDOW(burnergui->scrolledwindow));

  bCancel = false;
}

CDburner::~CDburner(){
  delete burnergui;
}

void CDburner::setGtkItems(GtkTextView* _textview, GtkLabel* _statuslabel, GtkButton* _donebutton, GtkButton* _cancelbutton, GtkScrolledWindow* _swindow){
  process_textview = _textview;
  gtk_text_view_set_buffer(GTK_TEXT_VIEW(process_textview), textbuffer);
  statuslabel = _statuslabel;
  donebutton = _donebutton;
  cancelbutton = _cancelbutton;
  scrollwindow = _swindow;
}

void CDburner::initBurnProcess(DiscView* _theDiscs){
  gtk_widget_show(burnergui->burndialog);
  gtk_window_set_transient_for(GTK_WINDOW(burnergui->burndialog), GTK_WINDOW(mainWindow));
  theDiscs = _theDiscs;
  currentDisc = -1;
  burnProcessDone = false;
  bCancel = false;
  if(setup->checkNeededExecs()){
    burnProcess(s_cancel);
  }
  else{
    burnProcess(s_start);
  }
}

void CDburner::burnProcess(int _state){
  if(_state != -1){
    currentState = _state;
  }

  switch(currentState){
  case s_start:
    burnStart();
    break;
  case s_newdisc:
    startNewDisc();
    break;
  case s_makeiso:
    setStatus("Making ISO image");
    makeISOfile();
    break;
  case s_blank:
    setStatus("Blanking Disc");
    blankDisc();
    break;
  case s_getdisc:
    setStatus("Insert Disc");
    requireDisc();
    break;
  case s_burning:
    setStatus("Burning Disc");
    burnDisc();
    break;
  case s_done:
    setStatus("Burn Process Done");
    burnDone();
    break;
  case s_cancel:
    setStatus("Burn Process Canceled");
    burnCancel();
    break;
  case s_nothing:
    break;
  }
}

void CDburner::setStatus(string _status){
  gtk_label_set_text(statuslabel, _status.c_str());
}

void CDburner::startNewDisc(){
  currentDisc++;
  if(currentDisc < theDiscs->allDiscs.size()){
    if(currentDisc > 0){
      setStatus("Deleting old ISO file");
      deleteProcessSetup();
      delfile.spawnProcess();
      theProcesses.addProcess(&delfile, s_makeiso);
    }
    else{
      burnProcess(s_makeiso);
    }
  }
  else{
    setStatus("Deleting old ISO file");
    deleteProcessSetup();
    delfile.spawnProcess();
    theProcesses.addProcess(&delfile, s_done);
  }
}

int CDburner::makeISOfile(){
  mkisofsProcessSetup(theDiscs->allDiscs[currentDisc]);
  mkisofs.spawnProcess();
  theProcesses.addProcess(&mkisofs, s_getdisc);
  return 0;
}

int CDburner::burnStart(){
  ejectProcessSetup();
  burnProcess(s_newdisc);
}

int CDburner::burnCancel(){
  theProcesses.killAll();
  burnProcessDone = true;
  return 0;
}

int CDburner::burnDone(){
  burnProcessDone = true;
  gtk_widget_set_sensitive(GTK_WIDGET(donebutton), true);
  gtk_widget_set_sensitive(GTK_WIDGET(cancelbutton), false);
  return 0;
}

int CDburner::blankDisc(){
  if((theDiscs->allDiscs[currentDisc]->discType == t_cdrw) && (setup->getSettingBool("cdr_blank_cdrw") == true)){
    blankProcessSetup();
    blank.spawnProcess();
    theProcesses.addProcess(&blank, s_burning);
  }
  else{
    burnProcess(s_burning);
  }
}

int CDburner::deleteProcessSetup(){
  string setting;
  
  delfile.resetProcess();
  delfile.clearArguments();
  delfile.setGtkTextItems(textbuffer, process_textview);  
  
  delfile.setPath("/bin/rm");
  delfile.addArgument("-v");
  
  delfile.addArgument(isofname);
}

int CDburner::blankProcessSetup(){
  string setting;
  
  blank.resetProcess();
  blank.clearArguments();
  blank.setGtkTextItems(textbuffer, process_textview);  

  setting = setup->getSettingString("cdrecordpath");
  blank.setPath(setting);

  blank.addArgument("-v");

  setting = "-dev=";
  setting += setup->getSettingString("burnerdev");
  blank.addArgument(setting);

  blank.addArgument("blank=minimal");
}

int CDburner::requireDisc(){
  bInsertdisc = true;
  if(!bCancel){
    if((setup->getSettingBool("cdr_eject_after_write") != true) || (currentDisc == 0)){
      eject.spawnProcess();
      eject.waitProcess();
    }
    insertdiscdialog(theDiscs->allDiscs[currentDisc]->getName());
    bInsertdisc = true;
  }
}

int CDburner::burnDisc(){
  cdrecordProcessSetup(isofname);
  cdrecord.spawnProcess();
  theProcesses.addProcess(&cdrecord, s_newdisc);
}

int CDburner::mkisofsProcessSetup(Disc* theDisc){
  string setting;
  
  mkisofs.resetProcess();
  mkisofs.clearArguments();
  mkisofs.setGtkTextItems(textbuffer, process_textview);  

  setting = setup->getSettingString("mkisofspath");
  mkisofs.setPath(setting);

  if(setup->getSettingBool("mkiso_rock_ridge") == true){
    mkisofs.addArgument("-r");
  }
  if(setup->getSettingBool("mkiso_joliet") == true){
    mkisofs.addArgument("-J");
  }
  if(setup->getSettingBool("mkiso_31_character_filenames") == true){
    mkisofs.addArgument("-l");
  }
  if(setup->getSettingBool("mkiso_allow_period_filenames") == true){
    mkisofs.addArgument("-L");
  }
  if(setup->getSettingBool("mkiso_joliet_long") == true){
    mkisofs.addArgument("-joliet-long");
  }
  
  setting = "-V";
  mkisofs.addArgument(setting);

  setting = theDisc->getName();
  mkisofs.addArgument(setting);

  setting = "-o";
  mkisofs.addArgument(setting);

  setting = setup->getSettingString("isotempdir");
  setting += "/";
  setting += theDisc->getName();
  setting += ".iso";
  mkisofs.addArgument(setting);
  isofname = setting;  

  mkisofs.addArgument("-graft-points");
  
  for(int i = 0; i < theDisc->vFolders.size(); i++){
    setting = theDisc->vFolders[i]->getBurnPath();
    setting += "=";
    setting += theDisc->vFolders[i]->getPath();
    mkisofs.addArgument(setting);
  }
}

int CDburner::cdrecordProcessSetup(string _isoname){
  string setting;

  cdrecord.resetProcess();
  cdrecord.clearArguments();
  cdrecord.setGtkTextItems(textbuffer, process_textview);  

  setting = setup->getSettingString("cdrecordpath");
  cdrecord.setPath(setting);

  cdrecord.addArgument("-v");

  setting = "-dev=";
  setting += setup->getSettingString("burnerdev");
  cdrecord.addArgument(setting);
  
  if(setup->getSettingBool("cdr_eject_after_write") == true){
    cdrecord.addArgument("-eject");
  }
  if(setup->getSettingBool("cdr_dao") == true){
    cdrecord.addArgument("-dao");
  }

  setting = "driveropts=";
  if(setup->getSettingBool("cdr_enable_burnproff") == true){
    setting += "burnfree";
  }
  else{
    setting += "noburnfree";
  }
  cdrecord.addArgument(setting);
     
  cdrecord.addArgument(_isoname);
}

int CDburner::ejectProcessSetup(){
  string setting;

  eject.resetProcess();
  eject.clearArguments();
  eject.setGtkTextItems(textbuffer, process_textview);  
  
  setting = setup->getSettingString("cdrecordpath");
  eject.setPath(setting);
  
  setting = "-dev=";
  setting += setup->getSettingString("burnerdev");
 
  eject.addArgument(setting);  
  eject.addArgument("-eject");

  eject.setCallbackState(false);
}
