#include "discutils.h"
#include "folderutils.h"
#include "fileutils.h"
#include "project.h"
#include "math.h"
#include "outofspacedialog.h"
#include "renamefolderdialog.h"
#include "editdiscdialog.h"
#include "maingui.h"

#include <iostream>
#include <string>
#include <sstream>

using namespace std;

extern GtkWidget *mainWindow;
extern Project *project;
extern MainGUI *maingui;

enum{
  FOLDERIMAGE_COLUMN1=0,
  FOLDER_COLUMN2=1
};

#define GLADE_HOOKUP_OBJECT(component,widget,name) \
  g_object_set_data_full (G_OBJECT (component), name, \
    gtk_widget_ref (widget), (GDestroyNotify) gtk_widget_unref)

Disc::Disc(){
  model = GTK_TREE_MODEL(gtk_list_store_new(4, GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING));
  discSizeMB = project->discView->newdiscsize;
  discSize = discSizeMB * pow(double(2), double(20));
  discSpaceLeft = discSize;
  discType = t_cdrw;
}

void Disc::removeFolders(){
  while(vFolders.size()){
    vFolders[0]->toggleSelectionState(false,true);
    removeFolder(vFolders[0]);
  }
}

Disc::~Disc(){
  g_object_unref(model);
}

int Disc::writeToFile(File *outfile){
  twinpair thepair;
  thepair.name = "disc";
  thepair.value = discName;
  outfile->write_pair(&thepair);

  for(int i = 0; i < vFolders.size(); i++){
    thepair.name = vFolders[i]->getPath();
    thepair.value = vFolders[i]->getBurnPath();
    outfile->write_pair(&thepair);
  }

  thepair.name = "discsize";
  stringstream convert;
  convert << discSizeMB << endl;
  string tmpStr;
  convert >> tmpStr;
  thepair.value = tmpStr;
  outfile->write_pair(&thepair);

  thepair.name = "disctype";
  switch(discType){
  case t_cdr:
    thepair.value = "CDR";
    break;
   case t_cdrw:
    thepair.value = "CDRW";
    break;
  case t_dvdr:
    thepair.value = "DVDR";
    break;    
  }
  outfile->write_pair(&thepair);

  return 0;
}

int Disc::createFromFile(File *infile){
  twinpair thepair;
  infile->read_pair(&thepair);
  if(thepair.name == "disc"){
    setName(thepair.value);
    
    infile->read_pair(&thepair);
    while((thepair.name != "discsize") && (!infile->fileeof())){
      Folder *tmpFolder = project->folderView->findPath(thepair.name);
      if(tmpFolder){
	addFolder(tmpFolder);
	tmpFolder->setBurnPath(thepair.value);
      }
      infile->read_pair(&thepair);
    }
    
    if(thepair.name == "discsize"){
      stringstream convert;
      convert << thepair.value << endl;
      double dSize;
      convert >> dSize;
      setSize(dSize, true);
    }
    else{
      return 1;
    }
    infile->read_pair(&thepair);
    if(thepair.name == "disctype"){
      if(thepair.value == "CDR"){
	setType(t_cdr);
      }
      if(thepair.value == "CDRW"){
	setType(t_cdrw);
      }
      if(thepair.value == "DVDR"){
	setType(t_dvdr);
      }
    }
    else{
      return 1;
    }    
  }
  else{
    return 1;
  }
  return 0;
}

char *Disc::getName(){
  return (char *)discName.c_str();
}

void Disc::setName(string _name){
  discName = _name;
  gtk_button_set_label(GTK_BUTTON(radiobutton), discName.c_str());
  updateAssignedFolders();
}

double Disc::getSize(){
  return discSize;
}

void Disc::setSize(double _size, bool _force){
  double oldSize = discSizeMB;
  discSizeMB = _size;
  discSize = discSizeMB * pow(double(2), double(20));
  updateProgressbar();
  if(discSpaceLeft < 0){
    if(!_force){
      discSizeMB = oldSize;
      discSize = discSizeMB * pow(double(2), double(20));
      updateProgressbar();
    }
    outofspacedialog(discName);
  }
}

void Disc::setType(int _type){
  discType = _type;
}

void Disc::updateAssignedFolders(){
  for(int i = 0; i < vFolders.size(); i++){
    vFolders[i]->updateAssignment(this);  
  }
}

void Disc::updateProgressbar(){
  double dSize = 0;
  for(int i = 0; i < vFolders.size(); i++){
    dSize += vFolders[i]->folderSize;
  }
  
  discSpaceLeft = discSize - dSize;
  double mSize = dSize / pow(double(2),double(20));
  stringstream convert;

  convert << (int)round(mSize) << endl;
  string barText;
  string tmpStr;
  convert >> tmpStr;
  barText = "(";
  barText += tmpStr;
  barText += " / ";
  convert << discSizeMB << endl;
  convert >> tmpStr;  
  barText += tmpStr;
  barText += "MB)";    

  gtk_progress_bar_set_text(GTK_PROGRESS_BAR (progressbar), barText.c_str());
  
  double dSize2 = discSize / 100;
  dSize /= dSize2;
  dSize /= 100;

  if(dSize > 1){
    dSize = 1;
  } 
  
  gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR (progressbar), dSize);

}

int Disc::haveSpaceForFolder(Folder *nFolder){
  if(nFolder->folderSize > discSpaceLeft){
    return 0;
  }
  return 1;
}

void Disc::addFolder(Folder *nFolder){

  bool bFound = false;
  for(int i = 0;i < vFolders.size(); i++){
    if(vFolders[i] == nFolder){
      bFound = true;
    }
  }
  
  if(!bFound){
    nFolder->assignToDisc(this);  
    vFolders.push_back(nFolder);
    for(int i = 0; i < nFolder->children.size(); i++){
      removeFolder(nFolder->children[i]);
      nFolder->children[i]->toggleSelectionState(false,true);
    }

    gtk_list_store_append(GTK_LIST_STORE(model), &iter);

    gtk_list_store_set(GTK_LIST_STORE(model), &iter,
		       FOLDERIMAGE_COLUMN1, project->folderimage, -1);
    
    string tmpStr = nFolder->getPath(); 
    tmpStr += " => ";
    string tmpStr2 = nFolder->getListPath(); 
    tmpStr += tmpStr2;
    nFolder->setBurnPath(tmpStr2);

    gtk_list_store_set(GTK_LIST_STORE(model), &iter,
		       FOLDER_COLUMN2, tmpStr.c_str() , -1);
    
    nFolder->iter2 = iter;

    updateProgressbar();
  }
}

void Disc::removeFolder(Folder *nFolder){
  int fIndex = -1;
  for(int i = 0;i < vFolders.size(); i++){
    if(vFolders[i] == nFolder){
      fIndex = i;
    }
  }
  
  if(fIndex != -1){
    stringstream convert;
    convert << fIndex;
    string tPath;
    convert >> tPath;
    GtkTreePath *thePath;
    thePath = gtk_tree_path_new_from_string(tPath.c_str());
    gtk_tree_model_get_iter(model, &iter, thePath);
    gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
    vFolders[fIndex]->assignToNone();
    vFolders.erase(vFolders.begin() + fIndex);
    updateProgressbar();
  }
}

void Disc::removeSelected(){
  if(gtk_tree_selection_get_selected(selection, &model, &iter)){
    Folder *tmpFolder = findIter(iter);
    tmpFolder->toggleSelectionState(false,true);
    removeFolder(tmpFolder);
  }
}

void Disc::setBurnPath(Folder *nFolder){
  string tmpStr2 = nFolder->getBurnPath();
  string tmpStr = nFolder->getPath(); 
  tmpStr += " => ";
  tmpStr += tmpStr2;
    
  gtk_list_store_set(GTK_LIST_STORE(model), &nFolder->iter2,
		     FOLDER_COLUMN2, tmpStr.c_str() , -1);

}


void Disc::renameSelected(string _burnPath){
  if(gtk_tree_selection_get_selected(selection, &model, &iter)){
    Folder *tmpFolder = findIter(iter);
    tmpFolder->setBurnPath(_burnPath);
    setBurnPath(tmpFolder);
  }
}

string Disc::getSelectedPath(){
  if(gtk_tree_selection_get_selected(selection, &model, &iter)){
    Folder *tmpFolder = findIter(iter);
    return tmpFolder->getBurnPath();
  }
  else{
    string sNone = "none";
    return sNone;
  }
}

Folder *Disc::findIter(GtkTreeIter _iter){
  Folder *tmpFolder = NULL;
  
  for(int i = 0; i < vFolders.size(); i++){
    if(vFolders[i]->iter2.user_data == _iter.user_data){
      tmpFolder = vFolders[i];
    }
  }
  return tmpFolder;
}

GtkWidget *Disc::create_disc (int discnumber){
  fixed = gtk_fixed_new ();
  gtk_widget_show (fixed);
  
  discframe = gtk_frame_new (NULL);
  gtk_widget_show (discframe);
  gtk_fixed_put (GTK_FIXED (fixed), discframe, 2, 2);
  gtk_widget_set_size_request (discframe, 490, 50);

  progressbar = gtk_progress_bar_new ();
  gtk_widget_show (progressbar);
  gtk_container_add (GTK_CONTAINER (discframe), progressbar);

  updateProgressbar();
  
  string tmpStr = "Disk";
  stringstream convert;
  convert << discnumber;
  string tmpStr2;
  convert >> tmpStr2;
  tmpStr += tmpStr2;
  
  discName = tmpStr;

  radiobutton = gtk_radio_button_new_with_mnemonic(NULL, tmpStr.c_str());
  gtk_widget_show (radiobutton);
  gtk_frame_set_label_widget(GTK_FRAME(discframe), radiobutton);
  gtk_radio_button_set_group(GTK_RADIO_BUTTON(radiobutton), project->discView->radiobutton_group);
  project->discView->radiobutton_group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(radiobutton));

  
  gtk_toggle_button_set_active((GtkToggleButton*)radiobutton, true);
  
  g_signal_connect ((gpointer) radiobutton, "clicked",
                    G_CALLBACK (radiobutton_clicked),
                    NULL);
  
  return fixed;
}

void Disc::applyModel(GtkTreeView* tree_view){
  gtk_tree_view_set_model(tree_view, model);
  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tree_view));
}

int DiscView::addDisc(bool _edit){
  discnumber++;
  curdiscnew = 1;

  Disc *tmpDisc = new Disc();
  GtkWidget *newDisc = tmpDisc->create_disc(discnumber);
  allDiscs.push_back(tmpDisc);
  tmpDisc->discSize = 700;
  tmpDisc->discType = t_cdrw;
  gtk_box_pack_start (GTK_BOX (vbox), newDisc, TRUE, TRUE, 0);
  
  for(int i = 0; i < allDiscs.size(); i++){
    allDiscs[i]->activeDisc = false;
  }
  
  tmpDisc->activeDisc = true; 
  tmpDisc->applyModel(tree_view);
  
  for(int i = 0; i < project->folderView->flatFolderList.size(); i++){
    project->folderView->flatFolderList[i]->toggleSelectionState(false,true);
  }

  guiSetActive(true);
  if(_edit){
    editDisc(tmpDisc);
  }
}

int DiscView::editDisc(){
  editDisc(getActiveDisc());
}
int DiscView::editDisc(Disc *_disc){
  editdiscdialog(_disc->getName(), _disc->discType, (int)_disc->discSizeMB);
  newdiscsize = (int)_disc->discSizeMB;
  newdisctype = _disc->discType;
  return 0;
}

int DiscView::removeDisc(){
  int active = -1;
  for(int i = 0; i < allDiscs.size(); i++){
    if(allDiscs[i]->activeDisc){
      active = i;;
    }
  }
  if(active != -1){
    gtk_container_remove((GtkContainer*)vbox,allDiscs[active]->fixed); 
    allDiscs[active]->removeFolders();
    delete allDiscs[active];
    allDiscs.erase(allDiscs.begin() + active);
    
    radiobutton_group = NULL;
    
    for(int i = 0; i < allDiscs.size(); i++){
      allDiscs[i]->activeDisc = false;
      gtk_radio_button_set_group (GTK_RADIO_BUTTON ( allDiscs[i]->radiobutton), radiobutton_group);
      radiobutton_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (allDiscs[i]->radiobutton));
    }
    
    if(allDiscs.size()){
      allDiscs[0]->activeDisc = true;
      gtk_toggle_button_set_active((GtkToggleButton*)allDiscs[0]->radiobutton, true);
    }
  }
  if(!allDiscs.size()){
    guiSetActive(false);
  }
}

void DiscView::resizeDiscs(int width){
  for(int i = 0; i < project->discView->allDiscs.size(); i++){
    gtk_widget_set_size_request(allDiscs[i]->discframe, width - 10 , 50);
  }
}

DiscView::DiscView(GtkTreeView *tree_view_disc, GtkWidget *vbox_disclist){  
  radiobutton_group = NULL;
  activebutton = NULL;
  discnumber = 0;
  curdiscnew = 0;
  newdiscsize = 700;
  tree_view = tree_view_disc;
  vbox = vbox_disclist;

  model = GTK_TREE_MODEL(gtk_list_store_new(4, GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING));

  folderimage_renderer = gtk_cell_renderer_pixbuf_new();
  folder_renderer = gtk_cell_renderer_text_new();

  folderimage_column = gtk_tree_view_column_new_with_attributes ("", folderimage_renderer, "pixbuf", 
							       FOLDERIMAGE_COLUMN1, NULL);
  folder_column = gtk_tree_view_column_new_with_attributes ("", folder_renderer, "text", 
							    FOLDER_COLUMN2, NULL);    
}

DiscView::~DiscView(){
  while(allDiscs.size()){
    gtk_container_remove((GtkContainer*)vbox,allDiscs[0]->fixed); 
    delete allDiscs[0];
    allDiscs.erase(allDiscs.begin());
  }
  guiSetActive(false);    
}

void DiscView::applyModel(){
  gtk_tree_view_set_model(tree_view, model);
  gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(tree_view), 0);
  
  gtk_tree_view_append_column(GTK_TREE_VIEW(tree_view), folderimage_column);
  gtk_tree_view_append_column(GTK_TREE_VIEW(tree_view), folder_column);
}

void DiscView::removeSelected(){
  Disc *tmpDisc = getActiveDisc();
  if(tmpDisc){
    tmpDisc->removeSelected();
  }
}

void DiscView::renameSelected(string _burnPath){
  Disc *tmpDisc = getActiveDisc();
  if(tmpDisc){
    tmpDisc->renameSelected(_burnPath);
  }
}

void DiscView::getSelectedPath(){
  Disc *tmpDisc = getActiveDisc();
  if(tmpDisc){
    string tmpStr = tmpDisc->getSelectedPath();
    if(tmpStr != "none"){
      renamefolderdialog(tmpStr);
    }
  }
}


Disc *DiscView::getActiveDisc(){
  Disc *aDisc = NULL;

  for(int i = 0; i < allDiscs.size(); i++){
    if(allDiscs[i]->activeDisc){
      aDisc = allDiscs[i];
    }
  }
  return aDisc;
}

void DiscView::guiSetActive(bool _state){
  gtk_widget_set_sensitive(maingui->removebutton, _state);
  gtk_widget_set_sensitive(maingui->renamebutton, _state);
  gtk_widget_set_sensitive(maingui->edit_disc_item, _state);
  gtk_widget_set_sensitive(maingui->delete_disc_item , _state);
  gtk_widget_set_sensitive(maingui->burn_discs_item, _state);
}

int DiscView::writeToFile(File *outfile){
  for(int i = 0; i < allDiscs.size(); i++){
    allDiscs[i]->writeToFile(outfile);
  }
  return 0;
}

int DiscView::createFromFile(File *infile){
  while(!infile->fileeof()){
    addDisc(false);
    if(allDiscs[allDiscs.size() -1]->createFromFile(infile)){
      removeDisc();
    }
  }
  curdiscnew = 0;
}

void radiobutton_clicked (GtkRadioButton *widget, gpointer user_data){

  if(gtk_toggle_button_get_active((GtkToggleButton*)widget)){
    project->discView->radioButtonEvent(widget);
  }
}


void DiscView::radioButtonEvent(GtkRadioButton *button){
  activebutton = button;
      
  for(int i = 0; i < allDiscs.size(); i++){
    allDiscs[i]->activeDisc = false;
    if(activebutton == (GtkRadioButton*)allDiscs[i]->radiobutton){
      allDiscs[i]->activeDisc = true;      
      allDiscs[i]->applyModel(tree_view);
      for(int ii = 0; ii < project->folderView->flatFolderList.size(); ii++){
	project->folderView->flatFolderList[ii]->toggleSelectionState(false,true);
      }
      for(int ii = 0; ii < allDiscs[i]->vFolders.size(); ii++){
	  allDiscs[i]->vFolders[ii]->toggleSelectionState(true,false);
      }
    }
  }
}
