#include "process.h"
#include <fstream>

gboolean read_input(GIOChannel *source, GIOCondition condition, gpointer data){
  ((Process*)data)->processInput(source);
  return 1;
}

gint my_s_pipe(gint fd[2]){
  return (socketpair(AF_UNIX, SOCK_STREAM, 0, fd));
}

Process::Process(){
  bUseCallback = true;
  resetProcess();
  
}

void Process::setGtkTextItems(GtkTextBuffer* _textbuffer, GtkTextView* _textview){
  textbuffer = _textbuffer; 
  gtk_text_buffer_set_text(textbuffer,"",0);
  offset = 0;
  linenum = 0;
  textview = _textview;
} 

void Process::setCallbackState(bool _state){
  bUseCallback = _state;
}

void Process::setPath(string _path){
  path = _path;
  
  string tmpStr = "/";
  
  string::size_type pos = path.rfind(tmpStr,path.size() -1);
  if(pos != string::npos){
    command = path.substr(pos + 1, path.size()-1);
  }
}

void Process::setCommand(string _command){
  command = _command;
}

void Process::addArgument(string arg){
  sArguments.push_back(arg);
}

void Process::clearArguments(){
  sArguments.clear();
}

void Process::resetProcess(){
  processDone = false;
  rValue = 0;    
  iotag_err = 0;
  iotag_out = 0;
}

int Process::spawnProcess(){
  processDone = false;
  rValue = 0;

  setupProcessIO();
  
  fcntl(in, F_SETFL, O_NONBLOCK);
  fcntl(out, F_SETFL, O_NONBLOCK);
  fcntl(err, F_SETFL, O_NONBLOCK);
  
  if(bUseCallback){
    setupProcessCB();
  }
  return 0;
}

int Process::waitProcess(){
  while(!checkProcess()){}
  return closeProcess();
}

int Process::killProcess(){
  kill(pid, SIGTERM);
  sleep(1);
  while(!checkProcess()){
    kill(pid, SIGKILL);  
  }
  return closeProcess();
}

int Process::checkProcess(){
  if(processDone){
    return 1;
  }
  else{
    int status;
    if((waitpid((pid_t)pid, &status, WNOHANG)) != 0){
      processDone = true;
      if(WIFEXITED(status)){
	rValue = WEXITSTATUS(status);
      }
      return 1;
    }
  }
  return 0;
}

int Process::closeProcess(){
  if(checkProcess()){
    
    if(bUseCallback){
      processInput(io_out);
      processInput(io_err);
    }
    
    close(in);
    close(out);
    close(err);
    
    if(iotag_err){
      g_source_remove(iotag_err);
    }
    if(iotag_out){
      g_source_remove(iotag_out);
    }
    
    return rValue;
  }
  else{
    return -1;
  }
}

int Process::processInput(GIOChannel *source){
  gchar buf;
  gchar *buf2;
  guint count;
  g_io_channel_read(source, &buf, 1, &count);
  if(count){
    if(buf == '\b'){
      iterold = iter;
      offset--;
      gtk_text_buffer_get_iter_at_line_index(textbuffer, &iter, linenum, offset);
      gtk_text_buffer_delete(textbuffer, &iter, &iterold);
    }
    else if(buf == '\r'){
      gtk_text_buffer_get_iter_at_line_index(textbuffer, &iter, linenum, offset);
      iterold = iter;
      offset = 0;
      gtk_text_buffer_get_iter_at_line_index(textbuffer, &iter, linenum, offset);
      gtk_text_buffer_delete(textbuffer, &iter, &iterold);
    }
    else {
      buf2 = g_convert(&buf, 1, "UTF-8", "ISO-8859-1", NULL, NULL, NULL);
      gtk_text_buffer_get_iter_at_line_index(textbuffer, &iter, linenum, offset);
      gtk_text_buffer_insert(textbuffer,&iter,buf2,-1);
      offset += strlen(buf2);
      g_free(buf2);
      
      GtkTextIter iter_start;        
      GtkTextIter iter_end;
      GtkTextMark *mark = NULL;
      gtk_text_buffer_get_bounds(textbuffer, &iter_start, &iter_end);
      mark = gtk_text_buffer_create_mark(textbuffer, NULL, &iter_end, false);
      gtk_text_view_scroll_mark_onscreen(textview, mark);      
      gtk_text_buffer_delete_mark(textbuffer, mark);      
    }
    if(buf == '\n') {
      linenum++;
      offset = 0;
    }
  }
  
  if(checkProcess()){
    do{ 
      g_io_channel_read(source, &buf, 1, &count);
      if(count){
	gtk_text_buffer_insert_at_cursor(textbuffer,&buf,1);
      }
    }while(count != 0);

    if(source == io_err){
      g_source_remove(iotag_err);
    }
    if(source == io_out){
      g_source_remove(iotag_out);
    }
  }
  return 1;
}

char **Process::buildArguments(){
  char **argu;
  argu = g_new0(char *, sArguments.size() + 2);
  argu[0] = new gchar[command.size()];
  strcpy(argu[0], command.c_str());
  
  for(int i = 1; i < sArguments.size() + 1; i++){
    argu[i] = new gchar[sArguments[i - 1].size()];
    strcpy(argu[i], sArguments[i - 1].c_str());
  }

  argu[sArguments.size() + 1] = NULL;
  
  return argu;
}

char *Process::buildArgumentList(){
  string argu = sArguments[0];
  
  for(int i = 1; i < sArguments.size(); i++){
    argu += " ";
    argu += sArguments[i];
  }

  char *argus = new char[argu.size()];
  strcpy(argus, argu.c_str());
  return argus;
}


void Process::freeArguments(char **_arguments){
    for(int i = 1; i < sArguments.size() + 1; i++){
      delete _arguments[i];
    }
    delete[] _arguments;
}

void Process::setupProcessIO(){
  gint fd1[2], fd2[2], fd3[2];
  gchar **arguments;  
  
  /* do the pipe-stuff */
  if (my_s_pipe(fd1) < 0 || 
      my_s_pipe(fd2) < 0 || 
      my_s_pipe(fd3) < 0) {
    g_error("pipe error\n");
  }
  
  if (( pid = fork()) < 0) {
    g_error("fork error\n");
  } else 
    if (pid > 0) {
      /* parent */
      close(fd1[0]);
      close(fd2[1]);
      close(fd3[1]);
      
      /* return new stdout/stdin/stderr of child */
      in = fd1[1];
      out = fd2[0];      
      err = fd3[0];
    } else {
	  
	  /* needed for correct rights in nonroot mode */
	  /* fix_guid(); */
	  
	  /* child */
	  close(fd1[1]);
	  close(fd2[0]);
	  close(fd3[0]);
	  
	  /* reroute stdin from child */
	  if (fd1[0] != STDIN_FILENO) {
	    if (dup2(fd1[0], STDIN_FILENO) != STDIN_FILENO) {
	      g_error("dup2 error on stdin\n");
	    }
	    close(fd1[0]);
	  }
	  /* reroute stdout from child */
	  if (fd2[1] != STDOUT_FILENO) {
	    if (dup2(fd2[1], STDOUT_FILENO) != STDOUT_FILENO) {
	      g_error("dup2 error on stdout\n");
	    }
	    close(fd2[1]);
	  }
	  /* reroute stderr from child */
	  if (fd3[1] != STDERR_FILENO) {
	    if (dup2(fd3[1], STDERR_FILENO) != STDERR_FILENO) {
	      g_error("dup2 error on stderr\n");
	    }
	    close(fd3[1]);
	  }
	  
	  /* startup child */
	  
	  arguments = buildArguments();
	  //cout << arguments;
	  //	  char *args[] = {"mkisofs","-o test.iso", "test.fil", NULL};
	  if (execv(path.c_str(),arguments)){
	    g_error("execv error\n");
	  }
	  //freeArguments(arguments);
	  
	}
}

void Process::setupProcessCB(){  
  io_out = g_io_channel_unix_new(out);
  io_err = g_io_channel_unix_new(err);
  iotag_err = g_io_add_watch(io_err, G_IO_IN, read_input, this);
  iotag_out = g_io_add_watch(io_out, G_IO_IN, read_input, this);
}
