#include "project.h"
#include "folderutils.h"
#include "discutils.h"
#include "setup.h"
#include "fileutils.h"
#include "saveprojectasdialog.h"
#include "maingui.h"

GtkWidget *mainWindow = NULL;
Project *project = NULL;
extern MainGUI *maingui;

Project::Project(){
  
  fimage = gtk_image_new_from_stock(GTK_STOCK_CDROM, GTK_ICON_SIZE_MENU);
  folderimage = gtk_widget_render_icon(fimage, GTK_STOCK_OPEN, GTK_ICON_SIZE_MENU, NULL); 

  cimage = gtk_image_new_from_stock(GTK_STOCK_OPEN, GTK_ICON_SIZE_MENU);
  cdromimage = gtk_widget_render_icon(cimage, GTK_STOCK_CDROM, GTK_ICON_SIZE_MENU, NULL);

  gtk_widget_set_sensitive(maingui->save_project_item, true);
  gtk_widget_set_sensitive(maingui->save_project_as_item, true);
  gtk_widget_set_sensitive(maingui->new_disc_item, true);
  
  projectFileName = "none";
  fileError = false;
  nextProjectState = 0;
}

void Project::newProject(char *_path){
  project->loadData(GTK_TREE_VIEW(maingui->treeview_folderlist), 
		    GTK_TREE_VIEW(maingui->treeview_discview), 
		    GTK_WIDGET(maingui->vbox_disclist), 
		    _path);
}

void Project::loadData(GtkTreeView* tree_view_folder, GtkTreeView* tree_view_disc, GtkWidget* vbox_disclist, char* _path){
  newFolderView(tree_view_folder, _path);
  newDiscView(tree_view_disc, vbox_disclist);
}

Project::~Project(){
  gtk_widget_set_sensitive(maingui->save_project_item, false);
  gtk_widget_set_sensitive(maingui->save_project_as_item, false);
  gtk_widget_set_sensitive(maingui->new_disc_item, false);


  if(!fileError){
    deleteFolderView();
    deleteDiscView();  
  }
}

int Project::writeToFile(){
  File *outfile;
  outfile = new File(projectFileName.c_str(), ios_base::out);
  if(outfile->fileok){
    twinpair thepair;
    thepair.name = "rootdir";
    thepair.value = baseDir->folderPath;
    outfile->write_pair(&thepair);
    discView->writeToFile(outfile);
    delete outfile;
    return 0;
  }
  return 1;
}

int Project::createFromFile(string _fileName){
  File *infile;
  infile = new File(_fileName.c_str(), ios_base::in);
  if(infile->fileok){
    projectFileName = _fileName;
    twinpair thepair;
    infile->read_pair(&thepair);
    if(thepair.name == "rootdir"){
      newProject((char *)thepair.value.c_str());
      discView->createFromFile(infile);
    }
    else{ 
      fileError = true;
      delete infile;
      return 1;
    }
    delete infile;
    return 0;
  }
  fileError = true;
  return 1;
}

int Project::save(){
  if(projectFileName == "none"){
    return saveas();
  }
  writeToFile();
  return 0;
}

int Project::saveas(){
  if(projectFileName == "none"){
    saveprojectasdialog("project1.dom");
  }
  else{
    saveprojectasdialog(projectFileName.c_str());
  }
  return 1;
}

int Project::deleteFolderView(){

  gtk_tree_view_remove_column(GTK_TREE_VIEW(folderView->tree_view), folderView->check_column);
  gtk_tree_view_remove_column(GTK_TREE_VIEW(folderView->tree_view), folderView->discimage_column);
  gtk_tree_view_remove_column(GTK_TREE_VIEW(folderView->tree_view), folderView->discassign_column);
  gtk_tree_view_remove_column(GTK_TREE_VIEW(folderView->tree_view), folderView->folderimage_column);
  gtk_tree_view_remove_column(GTK_TREE_VIEW(folderView->tree_view), folderView->folder_column);
  
  gtk_tree_view_set_model(folderView->tree_view, NULL);
  
  
  delete baseDir;
  delete folderView;
    
  return 0;
}

int Project::deleteDiscView(){
  
  gtk_tree_view_remove_column(GTK_TREE_VIEW(discView->tree_view), discView->folderimage_column);
  gtk_tree_view_remove_column(GTK_TREE_VIEW(discView->tree_view), discView->folder_column);

  gtk_tree_view_set_model(discView->tree_view, NULL);
  
  delete discView;
  return 0;
}

int Project::newFolderView(GtkTreeView *tree_view_folder, char *_path){
  baseDir = new Folder(_path);
  folderView = new FolderView(tree_view_folder);
  baseDir->fillModel(folderView->model);
  folderView->applyModel();
  folderView->createFlatList(baseDir);
  
  g_signal_connect(G_OBJECT (folderView->check_renderer), "toggled",
		   G_CALLBACK (toggle_event), NULL);
  return 0;
}

int Project::newDiscView(GtkTreeView *tree_view_disc, GtkWidget *vbox_disclist){
  discView = new DiscView(tree_view_disc, vbox_disclist);
  discView->applyModel();
  return 0;
}
