#include "setup.h"
#include "fileutils.h"
#include "process.h"
#include <fstream>
#include <iostream>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "filenotfounddialog.h"

Setup *setup;
extern GtkWidget *mainWindow;

Setup::Setup(){
  which = "none";
  setupDialog();
}

void Setup::createSetupValue(int _type, GtkWidget *_gui, string _name){
  SetupOption* newValue = new SetupOption(_type, _gui, _name);
  allOptions.push_back(newValue);
}

void Setup::setupDialog(){

  setupgui = new SetupGUI();
  setupdialog = setupgui->setupDialog();
  gtk_window_set_transient_for(GTK_WINDOW(setupdialog), GTK_WINDOW(mainWindow));
  
  GtkWidget *theGUI;

  theGUI = setupgui->cdr_path_entry;
  createSetupValue(SETUP_STRING, theGUI, "cdrecordpath");

  theGUI = setupgui->cdr_dev_entry;
  createSetupValue(SETUP_STRING, theGUI, "burnerdev");
  
  theGUI = setupgui->cdr_eject_checkbutton;
  createSetupValue(SETUP_BOOL, theGUI, "cdr_eject_after_write");

  theGUI = setupgui->cdr_blank_checkbutton;
  createSetupValue(SETUP_BOOL, theGUI, "cdr_blank_cdrw");

  theGUI = setupgui->cdr_burnproff_checkbutton;
  createSetupValue(SETUP_BOOL, theGUI, "cdr_enable_burnproff");

  theGUI = setupgui->cdr_dao_checkbutton;
  createSetupValue(SETUP_BOOL, theGUI, "cdr_dao");

  
  theGUI = setupgui->mkiso_path_entry;
  createSetupValue(SETUP_STRING, theGUI, "mkisofspath");

  theGUI = setupgui->mkiso_tmp_entry;
  createSetupValue(SETUP_STRING, theGUI, "isotempdir");

  theGUI = setupgui->mkiso_rr_checkbutton;
  createSetupValue(SETUP_BOOL, theGUI, "mkiso_rock_ridge");
    
  theGUI = setupgui->mkiso_jl_checkbutton;
  createSetupValue(SETUP_BOOL, theGUI, "mkiso_joliet");

  theGUI = setupgui->mkiso_31c_checkbutton;
  createSetupValue(SETUP_BOOL, theGUI, "mkiso_31_character_filenames");

  theGUI = setupgui->mkiso_ap_checkbutton;
  createSetupValue(SETUP_BOOL, theGUI, "mkiso_allow_period_filenames");

  theGUI = setupgui->mkiso_jll_checkbutton;
  createSetupValue(SETUP_BOOL, theGUI, "mkiso_joliet_long");

  
  theGUI = setupgui->burnbot_checkbutton;
  createSetupValue(SETUP_BOOL, theGUI, "burnbot_have_burnbot");
  burnbot_checkbutton = theGUI;

  theGUI = setupgui->burnbot_exe_entry;
  createSetupValue(SETUP_STRING, theGUI, "burnbotexe");
  burnbot_exe_entry = theGUI;

  theGUI = setupgui->burnbot_load_entry;
  createSetupValue(SETUP_STRING, theGUI, "burnbotloadcmd");
  burnbot_load_entry = theGUI;

  theGUI = setupgui->burnbot_unload_entry;
  createSetupValue(SETUP_STRING, theGUI, "burnbotunloadcmd");
  burnbot_unload_entry = theGUI;
}
 
void Setup::loadrcfile(){

  if(checkFile("/bin/which")){
    which = "/bin/which";
  }
  if(checkFile("/usr/local/bin/which")){
    which = "/usr/local/bin/which";
  }
  if(checkFile("/usr/bin/which")){
    which = "/usr/bin/which";
  }  

  read_rcfile();  
  set_gui_values();
  checkNeededExecs();
}

void Setup::set_gui_values(){

  for(int i = 0; i < allOptions.size(); i++){
    allOptions[i]->optionSetGUIValue();
  }

  toggleburnbot();
}

void Setup::update_settings(){
  
  for(int i = 0; i < allOptions.size(); i++){
    allOptions[i]->optionUpdateValue();
  }
  
  checkNeededExecs();
  
  write_rcfile();
  toggleburnbot();
    
}

void Setup::toggleburnbot(){
  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(burnbot_checkbutton))){
    gtk_widget_set_sensitive(burnbot_exe_entry, true);  
    gtk_widget_set_sensitive(burnbot_load_entry, true);  
    gtk_widget_set_sensitive(burnbot_unload_entry, true);  
  }
  else{
    gtk_widget_set_sensitive(burnbot_exe_entry, false);  
    gtk_widget_set_sensitive(burnbot_load_entry, false);  
    gtk_widget_set_sensitive(burnbot_unload_entry, false);      
  }
}

Setup::~Setup(){
  delete setupgui;
}

SetupOption* Setup::findSetupOption(string _name){
  SetupOption* theOption = NULL;
  for(int i = 0; i < allOptions.size(); i++){
    if(allOptions[i]->optionName == _name){
      theOption = allOptions[i];
    }
  }
  if(theOption == NULL){
    cout << "ERROR: option not fount!\n";
  }
  return theOption;
}

bool Setup::getSettingBool(string _name){
  SetupOption* theOption = NULL;
  theOption = findSetupOption(_name);
  if(theOption){
    return theOption->bSetupValue;
  }
  else{
    cout << "ERROR: Setting not found!\n";
    return -1;
  }
}

string Setup::getSettingString(string _name){
  SetupOption* theOption = NULL;
  theOption = findSetupOption(_name);
  if(theOption){
    return theOption->sSetupValue;
  }
  else{
    cout << "ERROR: Setting not found!\n";
    return "ERROR";
  }
}

int Setup::create_default_rcfile(){
  SetupOption *theOption;
  
  theOption = findSetupOption("cdrecordpath");

  string cdrecordpath;

  cdrecordpath = findFile("cdrecord");
  if(cdrecordpath == "error"){
    cdrecordpath = findFile("cdrecord.sh");
    if(cdrecordpath == "error"){
      cdrecordpath = "/usr/bin/cdrecord";
    }
  }
 
  theOption->optionSetValueString(cdrecordpath);

  theOption = findSetupOption("burnerdev");
  theOption->optionSetValueString("0,0,0");

  string mkisofspath;
  
  mkisofspath = findFile("mkisofs");
  if(mkisofspath == "error"){
    mkisofspath = "/usr/bin/mkisofs";
  }
  
  theOption = findSetupOption("mkisofspath");
  theOption->optionSetValueString(mkisofspath);

  theOption = findSetupOption("isotempdir");
  theOption->optionSetValueString("/tmp");
  

  theOption = findSetupOption("burnbotexe");
  theOption->optionSetValueString("/usr/bin/discbot");

  theOption = findSetupOption("burnbotloadcmd");
  theOption->optionSetValueString("load");

  theOption = findSetupOption("burnbotunloadcmd");
  theOption->optionSetValueString("unload");

  
  theOption = findSetupOption("cdr_eject_after_write"); 
  theOption->optionSetValueBool(true);

  theOption = findSetupOption("cdr_blank_cdrw");
  theOption->optionSetValueBool(true);

  theOption = findSetupOption("cdr_enable_burnproff"); 
  theOption->optionSetValueBool(true);

  theOption = findSetupOption("cdr_dao");
  theOption->optionSetValueBool(true);

  theOption = findSetupOption("mkiso_rock_ridge"); 
  theOption->optionSetValueBool(true);

  theOption = findSetupOption("mkiso_joliet");
  theOption->optionSetValueBool(true);

  theOption = findSetupOption("mkiso_31_character_filenames"); 
  theOption->optionSetValueBool(true);

  theOption = findSetupOption("mkiso_allow_period_filenames");
  theOption->optionSetValueBool(true);

  theOption = findSetupOption("mkiso_joliet_long");
  theOption->optionSetValueBool(false);

  theOption = findSetupOption("burnbot_have_burnbot");
  theOption->optionSetValueBool(false);
			    
  write_rcfile();
  checkNeededExecs();

  return 0;
}

int Setup::read_rcfile(){
  string configpath = getenv("HOME");
  configpath += "/.discomatic";
  ifstream rcfile(configpath.c_str());
  if(!rcfile){
    return create_default_rcfile();
  }
  
  while(!rcfile.eof()){
    read_pair(&rcfile);
  }

  for(int i = 0; i < configops.size(); i++){
    set_value(configops[i]);
  }

  return 0;
}

int Setup::set_value(twinpair thePair){

  SetupOption *theOption = findSetupOption(thePair.name);
  if(theOption){
    if(theOption->setupType == SETUP_STRING){
      theOption->optionSetValueString(thePair.value);
    }
    else{
      theOption->optionSetValueBool(string2bool(thePair.value));
    }
    return 0;
  }
  else{
    cout << "Error in config file!\n";
    return 1;
  }
}

bool Setup::string2bool(string tString){
  if(tString == "true"){
    return 1;
  }
  else{
    return 0;
  }
}

void Setup::read_pair(ifstream *file){
  char line[256];
  string tmpStr;
  file->getline(line,256);
  tmpStr = line;
 
  twinpair thepair;
  string findStr = "=";
  string::size_type pos = tmpStr.rfind(findStr,tmpStr.size() -1);
  if(pos != string::npos){
    thepair.name = tmpStr.substr(0,pos);
    thepair.value = tmpStr.substr(pos + 1, tmpStr.size() -1);
    configops.push_back(thepair);
  }
}

int Setup::write_rcfile(){
  string configpath = getenv("HOME");
  configpath += "/.discomatic";
  ofstream rcfile(configpath.c_str());
  if(!rcfile){
    cout << "Error writing config file!\n";
    return 1;
  }
  
  for(int i = 0; i < allOptions.size(); i++){
    allOptions[i]->optionWriteToFile(&rcfile); 
  }
}

int Setup::checkFile(string _filename){
  int rev = 0;
  int fd = open(_filename.c_str(), O_RDONLY);
  if(fd > 0){
    rev = 1;
    close(fd);
  }
  return rev;
}

int Setup::checkNeededExecs(){
  int rev = 0;
  string cdrecordpath = getSettingString("cdrecordpath");
  if(!checkFile(cdrecordpath)){
    BasicDialog *fnfdia = filenotfounddialog(cdrecordpath);
    fnfdia->setTransientFor(GTK_WINDOW(mainWindow));
    rev++;
  }
  string mkisofspath = getSettingString("mkisofspath");
  if(!checkFile(mkisofspath)){
    BasicDialog *fnfdia = filenotfounddialog(mkisofspath);
    fnfdia->setTransientFor(GTK_WINDOW(mainWindow));
    rev++;
  }

  if(getSettingBool("burnbot_have_burnbot")){
    string burnbotexe = getSettingString("burnbotexe");
    if(!checkFile(burnbotexe)){
      BasicDialog *fnfdia = filenotfounddialog(burnbotexe);
      fnfdia->setTransientFor(GTK_WINDOW(mainWindow));
      rev++;
    }
  }
  return rev;
}

string Setup::findFile(string _filename){
  if(which != "none"){
    Process fwhich;
    gchar buf[255];

    fwhich.setPath(which);
    fwhich.addArgument(_filename);
    fwhich.setCallbackState(false);
    
    fwhich.spawnProcess();
    while(!fwhich.checkProcess()){}
    ssize_t fdread = read(fwhich.out, &buf, 255);
    if(fwhich.closeProcess()){
      return "error";
    }
    buf[fdread - 1] = 0;
    string command = buf;
    return command;
  }
  else{
    return "error";
  }
}

SetupOption::SetupOption(int _type, GtkWidget *_gui, string _name){
  setupGUI = _gui;
  setupType = _type;
  optionName = _name;
}

void SetupOption::optionSetGUI(GtkWidget *_gui){
  setupGUI = _gui;
}

void SetupOption::optionSetValueBool(bool _state){
  bSetupValue = _state;
}

void SetupOption::optionSetValueString(string _value){
  sSetupValue = _value;
}

void SetupOption::optionSetGUIValue(){
  if(setupType == SETUP_STRING){
    gtk_entry_set_text(GTK_ENTRY(setupGUI), sSetupValue.c_str());
  }
  else{
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(setupGUI), bSetupValue); 
  }
}

void SetupOption::optionUpdateValue(){
  if(setupType == SETUP_STRING){
    sSetupValue = gtk_entry_get_text(GTK_ENTRY(setupGUI));
  }
  else{
    bSetupValue = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(setupGUI));
  }
}

void SetupOption::optionWriteToFile(ofstream *_file){
  if(setupType == SETUP_STRING){
    string tmpStr = optionName + "=" + sSetupValue + "\n";
    _file->write(tmpStr.c_str(), tmpStr.length());
  }
  else{
    string tmpStr = optionName + "=";
    if(bSetupValue)
      tmpStr += "true\n";
    else
      tmpStr += "false\n";
    _file->write(tmpStr.c_str(), tmpStr.length());
  }
}
